@@ -1,5 +1,6 @@
 from design_bench.datasets.dataset_builder import DatasetBuilder
 from design_bench.datasets.discrete_dataset import DiscreteDataset
+from design_bench.oracles.feature_extractors.feature_extractor import FeatureExtractor
 import abc
 import numpy as np
 import math
@@ -145,7 +146,9 @@ class OracleBuilder(abc.ABC):
                  internal_batch_size=32, internal_measurements=1,
                  noise_std=0.0, expect_normalized_y=False,
                  expect_normalized_x=False, expect_logits=None,
-                 max_samples=None, min_percentile=0.0, max_percentile=100.0):
+                 max_samples=None, distribution=None,
+                 min_percentile=0.0, max_percentile=100.0,
+                 feature_extractor: FeatureExtractor = None):
         """Initialize the ground truth score function f(x) for a model-based
         optimization problem, which involves loading the parameters of an
         oracle model and estimating its computational cost
@@ -185,12 +188,20 @@ class OracleBuilder(abc.ABC):
             the maximum number of samples to include in the visible dataset;
             if more than this number of samples would be present, samples
             are randomly removed from the visible dataset
+        distribution: Callable[np.ndarray, np.ndarray]
+            a function that accepts an array of the ranks of designs as
+            input and returns the probability to sample each according to
+            a distribution---for example, a geometric distribution
         max_percentile: float
             the percentile between 0 and 100 of prediction values 'y' above
             which are hidden from access by members outside the class
         min_percentile: float
             the percentile between 0 and 100 of prediction values 'y' below
             which are hidden from access by members outside the class
+        feature_extractor: FeatureExtractor
+            an optional feature extraction module that encodes designs and
+            scores from a model-based optimization dataset into different
+            feature spaces for training oracle models
 
         """
 
@@ -200,10 +211,6 @@ class OracleBuilder(abc.ABC):
             raise ValueError("is_logits is only defined "
                              "for use with discrete datasets")
 
-        # check the given dataset is compatible with this oracle
-        if not self.check_input_format(dataset):
-            raise ValueError("the given dataset is not compatible")
-
         # keep the dataset in case it is needed for normalization
         self.external_dataset = dataset
 
@@ -214,6 +221,7 @@ class OracleBuilder(abc.ABC):
         # draw statistics from a fixed distribution
         # this is necessary because self.dataset is mutable
         self.internal_dataset.subsample(max_samples=max_samples,
+                                        distribution=distribution,
                                         min_percentile=min_percentile,
                                         max_percentile=max_percentile)
 
@@ -231,11 +239,20 @@ class OracleBuilder(abc.ABC):
         self.is_batched = is_batched
         self.internal_batch_size = internal_batch_size
         self.internal_measurements = internal_measurements
+        self.feature_extractor = feature_extractor
 
         # attributes that describe model predictions
         self.noise_std = noise_std
         self.num_evaluations = self.external_dataset.dataset_size
 
+        # check the given dataset is compatible with this oracle
+        if not self.check_input_format(dataset):
+            raise ValueError("the given dataset is not compatible")
+
+        # update the name of this oracle
+        if self.feature_extractor is not None:
+            self.name = self.name + "_" + self.feature_extractor.name
+
     def dataset_to_oracle_x(self, x_batch, dataset=None):
         """Helper function for converting from designs contained in the
         dataset format into a format the oracle is expecting to process,
@@ -284,6 +301,10 @@ class OracleBuilder(abc.ABC):
         if self.expect_normalized_x:
             x_batch = self.internal_dataset.normalize_x(x_batch)
 
+        # apply a feature extraction process unique to the oracle
+        if self.feature_extractor is not None:
+            x_batch = self.feature_extractor\
+                .dataset_to_oracle_x(x_batch, self.internal_dataset)
         return x_batch
 
     def dataset_to_oracle_y(self, y_batch, dataset=None):
@@ -324,6 +345,10 @@ class OracleBuilder(abc.ABC):
         if self.expect_normalized_y:
             y_batch = self.internal_dataset.normalize_y(y_batch)
 
+        # apply a feature extraction process unique to the oracle
+        if self.feature_extractor is not None:
+            y_batch = self.feature_extractor\
+                .dataset_to_oracle_y(y_batch, self.internal_dataset)
         return y_batch
 
     def oracle_to_dataset_x(self, x_batch, dataset=None):
@@ -354,6 +379,11 @@ class OracleBuilder(abc.ABC):
         if dataset is None:
             dataset = self.external_dataset
 
+        # apply a feature extraction process unique to the oracle
+        if self.feature_extractor is not None:
+            x_batch = self.feature_extractor\
+                .oracle_to_dataset_x(x_batch, self.internal_dataset)
+
         # handle when the dataset is normalized and the normalization
         # statistics expected by the oracle are different
         if self.expect_normalized_x:
@@ -404,6 +434,11 @@ class OracleBuilder(abc.ABC):
         if dataset is None:
             dataset = self.external_dataset
 
+        # apply a feature extraction process unique to the oracle
+        if self.feature_extractor is not None:
+            y_batch = self.feature_extractor\
+                .oracle_to_dataset_y(y_batch, self.internal_dataset)
+
         # handle when the oracle expects normalized predictions but
         # the dataset is not currently normalized
         if self.expect_normalized_y:
